#!/usr/bin/env python3
# © Copyright 2020 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

"""
Python script to fetch the health status of Vigil for OES dashboard.
"""

import json
import subprocess
import sys
from collections import OrderedDict
from datetime import datetime

"""
service - Name of the service
properties - Systemctl properties
daemons - List of daemons. Any new daemon should be added here.
result - Final JSON of Vigil health status.
healthy/unhealthy/partially_healthy - Status of service
"""

service = "novell-vigil"
properties = ["ActiveState", "ActiveEnterTimestamp", "ConditionResult"]
kernel_modules = ["pmd", "vigil", "vigil_cifs", "vigil_ncp", "vigil_nss"]
result = OrderedDict([
    ("serviceName", "NSS Auditing"),
    ("status", "")])
healthy = "Healthy"
unhealthy = "Unhealthy"
partially_healthy = "Partially Healthy"

# The dict 'additional_info_key' contains the keys of resource bundle. The below 'keys' (ex: Err_Vigil_ServiceDown)
# will be used by dashboard app to get the localised string. 
additional_info_key = { "module_down": "Err_Vigil_ModuleDown", "service_down": "Common_ServiceInactive", "info_framework": "Info_Vigil_Framework" }

def get_systemctl_properties():
    """
    Function to run systemctl and capture the output.
    :return: dict: Result of systemctl
    """

    systemctl_result = dict()
    cmd = "systemctl show " + service + " --property=" + ",".join(properties)
    output = run_command(cmd)

    for line in output.splitlines():
        key, value = line.strip().split("=")
        systemctl_result[key] = value

    return systemctl_result


def is_daemon_running(process):
    """
    Function to check if the daemon process is running or not.
    :param process: Name of process
    :return: boolean: Status of process
    """

    cmd = "ps -ef | awk '$3 == 1' | grep -w '" + process + "' | wc -l"
    status = run_command(cmd)
    if int(status) > 0:
        return True
    else:
        return False


def generate_json(data):
    """
    Takes dict and converts to json.
    :param data: Data in dictionary form.
    :return: json: Converted value
    """

    formatted_data = json.dumps(data)
    return formatted_data


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """

    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate(timeout=5)
    except TimeoutError:
        ret.kill()

    if err:
        sys.exit(2)
    elif output:
        return output.decode('utf-8')


def update_result(state, uptime=0, info=""):
    """
    Function to update the result dictionary.
    :param state: State of the service (healthy/partially_healthy/unhealthy)
    :param uptime: Uptime of the service
    :param info: Additional information
    """

    result["status"] = state
    if state == healthy or state == partially_healthy:
        result.update({"runningSince" : uptime})
    if not info:
        info = additional_info_key["info_framework"]

    result.update({"AdditionalInfo" : info.split(",")})


def kernel_modules_status():
    """
    Inspects if all the modules are loaded.
    :return: boolean: status of modules
    """

    status = True
    cmd = "lsmod | grep -w -E '" + "|".join(kernel_modules) + "' | wc -l"
    if len(kernel_modules) != int(run_command(cmd)):
        status = False
    return status


def get_vigil_health():
    """
    The main method to get the status of Vigil. Here it checks novell-vigil service
    and related kernel modules loaded to decide the health.

    prints json object: Vigil health
    :return 0 on success
    """

    service_properties = get_systemctl_properties()
    if service_properties != -1:
        state = service_properties.get("ActiveState")
        timestamp = service_properties.get("ActiveEnterTimestamp")
        if timestamp:
            if timestamp == "n/a":
                uptime = 0
            else:
                uptime = int(datetime.strptime(timestamp, "%a %Y-%m-%d %H:%M:%S %Z").timestamp())

        if state == "active":
            if not kernel_modules_status():
                update_result(partially_healthy, uptime, info=additional_info_key["module_down"])
            else:
                update_result(healthy, uptime)
        else:
            condition_result = service_properties.get("ConditionResult")
            if condition_result == "yes":
                service_down = "Vigil," + additional_info_key["service_down"]
                update_result(unhealthy, info=service_down)
            else:
                return 0
    else:
        sys.exit(2)

    print(generate_json(result))
    return 0


# Script starts from here
if __name__ == '__main__':
    get_vigil_health()

