#!/usr/bin/env python3

# © Copyright 2020 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

"""
Python script to fetch Vigil data for OES telemetry.
"""

import glob
import json
import os
import subprocess
import sys
import time
from collections import OrderedDict

"""
service - Name of the service
properties - Systemctl properties
result - Final result format of Vigil telemetry data.
telemetry_result_loc - Location to place DFS data for telemetry
telemetry_config - Telemetry config file's location
"""

service = "novell-vigil"
properties = ["ActiveState"]
result = OrderedDict([
    ("OESService", "NSS Auditing"),
    ("timestamp", int(time.time()))])
telemetry_result_loc = "/var/opt/novell/telemetry/data/"
telemetry_config = "/etc/opt/novell/telemetry/config/telemetry.json"


def get_systemctl_properties():
    """
    Function to run systemctl and capture the output.
    :return: dict: Result of systemctl
    """

    systemctl_result = dict()
    cmd = "systemctl show " + service + " --property=" + ",".join(properties)
    output = run_command(cmd)

    for line in output.splitlines():
        key, value = line.strip().split("=")
        systemctl_result[key] = value

    return systemctl_result


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """

    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate(timeout=5)
    except TimeoutError:
        ret.kill()

    if err:
        sys.exit(2)
    elif output:
        return output.decode('utf-8')


def add_error(msg):
    """
    Function to add error to result
    :param msg: Error message
    """

    if "Error" in result:
        result["Error"] += [msg]
    else:
        result["Error"] = [msg]


def add_clients_info():
    """
    Function to add client information to result.
    """
    cmd = "ls /sys/audit/vigil/ | grep '^CLIENT_' | wc -l"
    no_of_clients = run_command(cmd)
    result["Registered Clients"] = int(no_of_clients)


def add_vlog_format():
    """
    Function to add vlog registered formats.
    """
    cmd = "/usr/bin/ps -ef | /usr/bin/grep -w vlog | /usr/bin/grep -v grep"
    vlog_result = run_command(cmd)

    if vlog_result:
        vlogs = vlog_result.splitlines()
        vlog_formats = []
        for vlog in vlogs:

            vlog_array = vlog.split()

            if '-f' in vlog_array:
                index = vlog_array.index('-f')
                vformat = vlog_array[index + 1]
            else:
                vformat = "XML"
            vlog_formats.append(vformat.upper())

        result["Requested Format"] = ",".join(list(set(vlog_formats)))


def write_result():
    """
    Function to remove old, create new and update the file with result.
    """

    files = glob.glob(telemetry_result_loc + 'Vigil_*.json')
    for file in files:
        try:
            os.remove(file)
        except OSError as e:
            add_error("Unable to delete old data file: " + e.strerror)

    file_name = "Vigil_" + str(result["timestamp"]) + ".json"
    file_path = telemetry_result_loc + file_name
    file = open(file_path, "w")
    file.write(json.dumps(result))
    file.close()


def telemetry_check():
    """
    To check if telemetry is enabled.
    :return: Boolean
    """

    try:
        with open(telemetry_config, "r") as file:
            status = json.load(file)
            if status["opt-in"].lower() == "yes":
                return True
    except IOError:
        print("Telemetry config file is not accessible")

    return False


def generate_vigil_data():
    """
    The main Function to get the data of Vigil for Telemetry.
    :return 0
    """

    # generate only if telemetry is enabled
    enabled = telemetry_check()
    if enabled:
        service_properties = get_systemctl_properties()
        if service_properties != -1:
            state = service_properties.get("ActiveState")
            if state == "active":
                add_clients_info()
                add_vlog_format()
        else:
            add_error("systemctl returned -1")
        write_result()
    return 0


# Script starts from here
if __name__ == '__main__':
    generate_vigil_data()
